;+
; NAME:
;
;   SHOCKPLOT
;
; VERSION:
;
;   1.01 (April 17, 2008)
;
; PURPOSE:
;
;   Interactivle plots of grids of MAPPINGS III shock models on emission 
;   line ratio diagrams. Line ratio axes can be constructed from a full
;   line list, and standard line ratio diagram axes are provided. Model
;   grids are plotted as lines of constant shock velocity, and lines of
;   constant magnetic field. In addition to interactive plotting, an IDL
;   command line command can be reported which allows reconstruction of
;   the same plot from the IDL prompt (and combined with user commands
;   for example to include data points).
;
; REFERENCE:
; 
;   'The MAPPINGS III Library of Fast Shock Models'
;    Allen, M. G. et al. 2008, ApJS - submitted
;
; AUTHOR:
;
;   Mark G. Allen
;   Observatoire de Strasbourg
;   11 Rue de l'Universite
;   Strasbourg 67000, FRANCE
;   E-mail: allen@astro.u-strasbg.fr
; 
; CONTIBUTORS:
; 
;   Brent A. Groves
;   Sterrewacht Leiden, Leiden University, The Netherlands
;
; CATEGORY:
;
;   Theoretical Model Plotting, Widgets 
;
; CALLING SEQUENCE:
;
;   shockplot      
;
; INPUTS:
;
;    None
;
; KEYWORD PARAMETERS:
;
;    None
;
; OUTPUT:
;
;    Postscript files of the generated plots
;
; COMMON BLOCKS:
; 
;    None
;
; RESTRICTIONS:
;
;   Requires all the programs in the SHOCKPLOT package. 
;   This includes a number of programs from: 
;
;      i) FANNING SOFTWARE CONSULTING Coyote Library: 
;      http://www.dfanning.com/programs/error_message.pro
;      http://www.dfanning.com/programs/pickcolorname.pro
;      http://www.dfanning.com/programs/ctload.pro
;      http://www.dfanning.com/programs/fsc_color.pro
;      http://www.dfanning.com/programs/number_formatter.pro
;      http://www.dfanning.com/programs/colorbar.pro
;      (see LICENSE)
;
;      ii) C D Pike, RAL:
;      tag_exist.pro
;
;      iii) Dominic Zarro (ARC):
;      rem_tag.pro
;
;      iv) Piero Rosati (ESO):
;      ps_start.pro
;      ps_end.pro
;
;      v) JHUAPL library
;       datatype.pro 
;
;   Requires all the model data file in the save_files/ directory
;
;
; MODIFICATION HISTORY:
;
;  2006-02-21 SHOCKPLOT v0.1
;  2008-03-04 SHOCKPLOT v1.0 First major release to coincide with finalising
;             the 'The MAPPINGS III Library of Fast Shock Models' paper. 
;             Re-organised the interface, adding a standard menu bar.
;             Restricted the model set to shock models only. 
;             Added colour plotting of the model grids
;  2008-04-17 SHOCKPLOT v1.01 Minor improvements over v1.0
;             The procedure datatype.pro was added as it is needed by 
;             tag_exist.pro . The Xmanager now uses a new procuedure
;             Shockplot_Cleanup to free up the memory used by Heap Variables 
;             from all the pointers, and prevents memory leakage.
;                          
;
;##############################################################################
;
; LICENSE
;
;  Copyright (C) 2008 M. G. Allen
;
;  SHOCKPLOT is free software; you can redistribute it and/or modify it.
; 
;  Those components from other authors as listed above have licenses indicated
;  in the indivdual program files.
;
;
;##############################################################################


PRO file_menu_handler, event

; Handle events from the File button in the menu


; Get the UValue of activated widget
widget_control, event.id, get_uvalue=uvalue

CASE uvalue of
'exit': begin
         bye_events,event
         return
     end
'print': begin
         ps_events,event
         return
     end
endcase
END ; of  file_menu_handler ===================================================



PRO options_menu_handler, event

; Handle events from the Options button in the menu

; get uvalue of activated widget
widget_control, event.id, get_value=value
widget_control, event.id, get_uvalue=uvalue
Widget_Control, event.top, Get_UValue=info

case value of
'Velocity Labels': begin
         if uvalue EQ 0 then begin
         v=(indgen(10)+1) * 100
         *info.vlabs = v
         Widget_Control, event.id, Set_Button=1
         Widget_control, event.id, Set_UValue=1
     endif else begin
         *info.vlabs = 0
         Widget_Control, event.id, Set_Button=0
         Widget_control, event.id, Set_UValue=0
     endelse
 end
'Magnetic Field Labels': begin
         if uvalue EQ 0 then begin
         *info.blabs = 1
         Widget_Control, event.id, Set_Button=1
         Widget_control, event.id, Set_UValue=1
     endif else begin
         *info.blabs = 0
         Widget_Control, event.id, Set_Button=0
         Widget_control, event.id, Set_UValue=0
     endelse
end
'Print Grid Values': begin
         if uvalue EQ 0 then begin
         *info.p_grid = 1
         Widget_Control, event.id, Set_Button=1
         Widget_control, event.id, Set_UValue=1
     endif else begin
         *info.p_grid = 0
         Widget_Control, event.id, Set_Button=0
         Widget_control, event.id, Set_UValue=0
     endelse
end
'Print IDL Command': begin
         if uvalue EQ 0 then begin
         *info.p_command = 1
         Widget_Control, event.id, Set_Button=1
         Widget_control, event.id, Set_UValue=1
     endif else begin
         *info.p_command = 0
         Widget_Control, event.id, Set_Button=0
         Widget_control, event.id, Set_UValue=0
     endelse
end
'Colour Key': begin
         if uvalue EQ 0 then begin
         *info.plot_ct = 1
         Widget_Control, event.id, Set_Button=1
         Widget_control, event.id, Set_UValue=1
     endif else begin
         *info.plot_ct = 0
         Widget_Control, event.id, Set_Button=0
         Widget_control, event.id, Set_UValue=0
     endelse
end

     endcase

Widget_Control, event.top, Set_UValue=info

END  ; of options_menu_handler ===============================================


PRO linelist_menu_handler, event

; Handle events from the Emission-Line-List button in the menu

; Get the UValue of activated widget
widget_control, event.id, get_uvalue=uvalue
Widget_Control, event.top, Get_UValue=info

full_line_list = *info.full_line_list
; construct a string array of the array names for display in the line
; selection widgets
lines = STRCOMPRESS(full_line_list.a +' '+ full_line_list.i + string(full_line_list.w))
; construct an integer array of the indexes of the lines (which is the way the
; lines are identified explicitly in the plotting routines (plot_grid)
nlines = indgen(N_ELEMENTS(lines))


CASE uvalue of
'full': line_selection = indgen(n_elements(lines))
;......................................................................................
's1': line_selection = [1109, 1112, 1124, 1126, 1157, $
      1160, 1187, 1188, 1189, 1190, 1191, 1192, 1193, $
      1194, 1195, 1196, 1197, 1198, 1204, 1209, 1210, $
      1218, 1220, 1221, 1226, 1227, 1228, 1247, 1248, $
      1277, 1303, 1331, 1354, 1345, 1346, 1368, 1369, $
      1377, 1384, 1393, 1400, 1404, 1405, 1418, 1422, $
      1434, 1435, 1439, 1440, 1453, 1454, 1456, 1457, $
      1498, 1507, 1626, 1637, 1658, 1670, 1664, 1675, $
      1686, 1690, 1694, 1698, 1709 ]
;......................................................................................
'wr1': line_selection =WHERE((full_line_list.w GT 1000) AND (full_line_list.w LT 2000))
;......................................................................................
'wr2': line_selection =WHERE((full_line_list.w GT 2000) AND (full_line_list.w LT 3000))
;......................................................................................
'wr3': line_selection =WHERE((full_line_list.w GT 3000) AND (full_line_list.w LT 4000))
;......................................................................................
'wr4': line_selection =WHERE((full_line_list.w GT 4000) AND (full_line_list.w LT 5000))
;......................................................................................
'wr5': line_selection =WHERE((full_line_list.w GT 5000) AND (full_line_list.w LT 6000))
;......................................................................................
'wr6': line_selection =WHERE((full_line_list.w GT 6000) AND (full_line_list.w LT 7000))
;......................................................................................
'wr7': line_selection =WHERE((full_line_list.w GT 7000) AND (full_line_list.w LT 8000))
;......................................................................................
'wr8': line_selection =WHERE((full_line_list.w GT 8000) AND (full_line_list.w LT 9000))
;......................................................................................
'wr9': line_selection =WHERE((full_line_list.w GT 9000) AND (full_line_list.w LT 10000))
;......................................................................................
'wr10': line_selection =WHERE((full_line_list.w GT 10000) AND (full_line_list.w LT 20000))
;......................................................................................
'wr11': line_selection =WHERE((full_line_list.w GT 20000) AND (full_line_list.w LT 50000))
;......................................................................................
'wr12': line_selection =WHERE((full_line_list.w GT 50000) AND (full_line_list.w LT 100000))
;......................................................................................
'wr13': line_selection =WHERE((full_line_list.w GT 100000) AND (full_line_list.w LT 250000))
;......................................................................................
'wr14': line_selection =WHERE((full_line_list.w GT 250000) AND (full_line_list.w LT  1000000))
;......................................................................................
'wr15': line_selection =WHERE((full_line_list.w GT 1000000) AND (full_line_list.w LT 2000000))
;......................................................................................
'wr16': line_selection =WHERE((full_line_list.w GT 2000000) AND (full_line_list.w LT 3000000))

ENDCASE

; use the line selection to make the options and option values for value and
; UValues of the line selection widgets
options = lines[line_selection]
option_vals = nlines[line_selection]

; re-set all the line selection widgets with the new line list
Widget_Control, info.y1_id, Set_Value = options
Widget_Control, info.y1_id, Set_UValue = option_vals
Widget_Control, info.y2_id, Set_Value = options
Widget_Control, info.y2_id, Set_UValue = option_vals
Widget_Control, info.x1_id, Set_Value = options
Widget_Control, info.x1_id, Set_UValue = option_vals
Widget_Control, info.x2_id, Set_Value = options
Widget_Control, info.x2_id, Set_UValue = option_vals

END ; of  linelist_menu_handler ===================================================


PRO help_menu_handler, event

; Handle events from the Help button in the menu

Widget_Control, event.top, Get_UValue=info
widget_control, event.id, get_uvalue=uvalue

case uvalue of
'about': begin
         about_text = ['SHOCKPLOT version 1.0', $
                        'Mark G. ALLEN, March 4, 2008']
         ok = dialog_message(about_text,/information, dialog_parent=info.MSID)
         return
     end
'instructions': begin
         xdisplayfile, 'INSTRUCTIONS', height=32
         return
     end
'readme': begin
         xdisplayfile, 'README', height=32
         return
     end
endcase
END ; of help_menu_handler ===================================================



PRO std_diags_menu_handler, event

; Handle events of selecting one of the standard diagrams listed under the 
; Diagrams button on the main menu. This constructs and sends pseudo events
; to the X1, X2, Y1, Y2 line selection widgets, mimicing the selection of
; the appropriate lines, just as if they'd been selected in those lists

Widget_Control, event.id, Get_UValue=val
Widget_Control, event.top, Get_UValue=info
;
; Send a pseudo event to the linelist_menu_handler to select the full linelist
; to ensure that the lines in the stadard diagram selected will be available
; in the line list
linelist_menu_handler, {ID:info.fullID, Top:event.top} 
;
; indicate that something has changed needing a new plot
info.lim_change=1

; construct and send pseudo events to line selection widgets
x1_id = info.x1_id
Widget_Control, x1_id, Get_UValue=x1_option_vals
Clear_X1_events, {ID:info.x1_clear, Top:event.top}
FOR i=0,n_elements(val.x1)-1 DO BEGIN
x1_event_index = where(x1_option_vals EQ val.x1[i])
pseudo_event = {ID:x1_id, Top:event.top, Handler:0L, Index:x1_event_index, Clicks:1L}
X1_events, pseudo_event
ENDFOR
;
x2_id = info.x2_id
Widget_Control, x2_id, Get_UValue=x2_option_vals
Clear_X2_events, {ID:info.x2_clear, Top:event.top}
FOR i=0,n_elements(val.x2)-1 DO BEGIN
x2_event_index = where(x2_option_vals EQ val.x2[i])
pseudo_event = {ID:x2_id, Top:event.top, Handler:0L, Index:x2_event_index, Clicks:1L}
X2_events, pseudo_event
ENDFOR
;
y1_id = info.y1_id
Widget_Control, y1_id, Get_UValue=y1_option_vals
Clear_Y1_events, {ID:info.y1_clear, Top:event.top}
FOR i=0,n_elements(val.y1)-1 DO BEGIN
y1_event_index = where(y1_option_vals EQ val.y1[i])
pseudo_event = {ID:y1_id, Top:event.top, Handler:0L, Index:y1_event_index, Clicks:1L}
Y1_events, pseudo_event
ENDFOR
;
y2_id = info.y2_id
Widget_Control, y2_id, Get_UValue=y2_option_vals
Clear_Y2_events, {ID:info.y2_clear, Top:event.top}
FOR i=0,n_elements(val.y2)-1 DO BEGIN
y2_event_index = where(y2_option_vals EQ val.y2[i])
pseudo_event = {ID:y2_id, Top:event.top, Handler:0L, Index:y2_event_index, Clicks:1L}
Y2_events, pseudo_event
ENDFOR
;
Widget_Control, event.top, Set_UValue=info
END ; of std_diags_menu_handler ==============================================


PRO X1_events, event

; Handle events from the X1 widget which selects emission lines 

Widget_Control, event.id, Get_UValue=option_vals
Widget_Control, event.top, Get_UValue=info

; line list index of the selected line
val = option_vals(event.index)

; Cumulative line list index widget 
Widget_Control, info.X1_selected, Get_UValue=x1_selected_uv   ; integer array

; extract the arrays from the user value structure
l_list_index = x1_selected_uv.l_list_index
option_vals_index = x1_selected_uv.option_vals_index 

; update the selected line list indicies
IF l_list_index[0] EQ -1 THEN BEGIN
   l_list_index_new = [val]
   option_vals_index_new = event.index
ENDIF ELSE BEGIN
   l_list_index_new = [l_list_index, val] 
   option_vals_index_new = [option_vals_index, event.index] 
ENDELSE

; re-define the string array of the selected lines
line_str = info.l_list[l_list_index_new]

; re-define the selected lines widget user value structure, and value  
x1_selected_uv_new = {l_list_index:l_list_index_new, option_vals_index:option_vals_index_new}
x1_selected_v_new = line_str

; set the widgets with the new values
Widget_Control, info.X1_selected, Set_Value= x1_selected_v_new
Widget_Control, info.X1_selected, Set_UValue=x1_selected_uv_new

; update the info structure with the line list indicies
; of the lines to be plotted on the axis.
*info.x1=l_list_index_new

; indicate that something has changed needing a new plot
info.lim_change=1

Widget_Control, event.id, Set_List_Select=option_vals_index_new
Widget_Control, event.top, Set_UValue=info

END  ; of X1_events ==========================================================


PRO X2_events, event

; Handle events from the X2 widget which selects emission lines 

Widget_Control, event.id, Get_UValue=option_vals
Widget_Control, event.top, Get_UValue=info

; line list index of the selected line
val = option_vals(event.index)

; Cumulative line list index widget 
Widget_Control, info.X2_selected, Get_UValue=x2_selected_uv   ; integer array

; extract the arrays from the user value structure
l_list_index = x2_selected_uv.l_list_index
option_vals_index = x2_selected_uv.option_vals_index 

; update the selected line list indicies
IF l_list_index[0] EQ -1 THEN BEGIN
   l_list_index_new = [val]
   option_vals_index_new = event.index
ENDIF ELSE BEGIN
   l_list_index_new = [l_list_index, val] 
   option_vals_index_new = [option_vals_index, event.index] 
ENDELSE

; re-define the string array of the selected lines
line_str = info.l_list[l_list_index_new]

; re-define the selected lines widget user value structure, and value  
x2_selected_uv_new = {l_list_index:l_list_index_new, option_vals_index:option_vals_index_new}
x2_selected_v_new = line_str

; set the widgets with the new values
Widget_Control, info.X2_selected, Set_Value= x2_selected_v_new
Widget_Control, info.X2_selected, Set_UValue=x2_selected_uv_new

; update the info structure with the line list indicies
; of the lines to be plotted on the axis.
*info.x2=l_list_index_new

; indicate that something has changed needing a new plot
info.lim_change=1

Widget_Control, event.id, Set_List_Select=option_vals_index_new
Widget_Control, event.top, Set_UValue=info

END  ; of X2_events ==========================================================


PRO Y1_events, event

; Handle events from the Y1 widget which selects emission lines 

Widget_Control, event.id, Get_UValue=option_vals
Widget_Control, event.top, Get_UValue=info

; line list index of the selected line
val = option_vals(event.index)

; Cumulative line list index widget 
Widget_Control, info.Y1_selected, Get_UValue=y1_selected_uv   ; integer array

; extract the arrays from the user value structure
l_list_index = y1_selected_uv.l_list_index
option_vals_index = y1_selected_uv.option_vals_index 

; update the selected line list indicies
IF l_list_index[0] EQ -1 THEN BEGIN
   l_list_index_new = [val]
   option_vals_index_new = event.index
ENDIF ELSE BEGIN
   l_list_index_new = [l_list_index, val] 
   option_vals_index_new = [option_vals_index, event.index] 
ENDELSE

; re-define the string array of the selected lines
line_str = info.l_list[l_list_index_new]

; re-define the selected lines widget user value structure, and value  
y1_selected_uv_new = {l_list_index:l_list_index_new, option_vals_index:option_vals_index_new}
y1_selected_v_new = line_str

; set the widgets with the new values
Widget_Control, info.Y1_selected, Set_Value= y1_selected_v_new
Widget_Control, info.Y1_selected, Set_UValue=y1_selected_uv_new

; update the info structure with the line list indicies
; of the lines to be plotted on the axis.
*info.y1=l_list_index_new

; indicate that something has changed needing a new plot
info.lim_change=1

Widget_Control, event.id, Set_List_Select=option_vals_index_new
Widget_Control, event.top, Set_UValue=info

END  ; of Y1_events ==========================================================


PRO Y2_events, event

; Handle events from the Y2 widget which selects emission lines 

Widget_Control, event.id, Get_UValue=option_vals
Widget_Control, event.top, Get_UValue=info

; line list index of the selected line
val = option_vals(event.index)

; Cumulative line list index widget 
Widget_Control, info.Y2_selected, Get_UValue=y2_selected_uv   ; integer array

; extract the arrays from the user value structure
l_list_index = y2_selected_uv.l_list_index
option_vals_index = y2_selected_uv.option_vals_index 

; update the selected line list indicies
IF l_list_index[0] EQ -1 THEN BEGIN
   l_list_index_new = [val]
   option_vals_index_new = event.index
ENDIF ELSE BEGIN
   l_list_index_new = [l_list_index, val] 
   option_vals_index_new = [option_vals_index, event.index] 
ENDELSE

; re-define the string array of the selected lines
line_str = info.l_list[l_list_index_new]

; re-define the selected lines widget user value structure, and value  
y2_selected_uv_new = {l_list_index:l_list_index_new, option_vals_index:option_vals_index_new}
y2_selected_v_new = line_str

; set the widgets with the new values
Widget_Control, info.Y2_selected, Set_Value= y2_selected_v_new
Widget_Control, info.Y2_selected, Set_UValue=y2_selected_uv_new

; update the info structure with the line list indicies
; of the lines to be plotted on the axis.
*info.y2=l_list_index_new

; indicate that something has changed needing a new plot
info.lim_change=1

Widget_Control, event.id, Set_List_Select=option_vals_index_new
Widget_Control, event.top, Set_UValue=info

END  ; of Y2_events ==========================================================


PRO Clear_X1_events, event
Widget_Control, event.top, Get_UValue=info
Widget_Control, info.X1_selected, Set_Value=''
Widget_Control, info.X1_selected, Set_UValue={l_list_index:[-1], option_vals_index:[-1]}
Widget_Control, info.x1_id, Set_List_Select=-1
*info.x1=-1
Widget_Control, event.top, Set_UValue=info
END

PRO Clear_X2_events, event
Widget_Control, event.top, Get_UValue=info
Widget_Control, info.X2_selected, Set_Value=''
Widget_Control, info.X2_selected, Set_UValue={l_list_index:[-1], option_vals_index:[-1]}
Widget_Control, info.x2_id, Set_List_Select=-1
*info.x2=-1
Widget_Control, event.top, Set_UValue=info
END

PRO Clear_Y1_events, event
Widget_Control, event.top, Get_UValue=info
Widget_Control, info.Y1_selected, Set_Value=''
Widget_Control, info.Y1_selected, Set_UValue={l_list_index:[-1], option_vals_index:[-1]}
Widget_Control, info.y1_id, Set_List_Select=-1
*info.y1=-1
Widget_Control, event.top, Set_UValue=info
END

PRO Clear_Y2_events, event
Widget_Control, event.top, Get_UValue=info
Widget_Control, info.Y2_selected, Set_Value=''
Widget_Control, info.Y2_selected, Set_UValue={l_list_index:[-1], option_vals_index:[-1]}
Widget_Control, info.y2_id, Set_List_Select=-1
*info.y2=-1
Widget_Control, event.top, Set_UValue=info
END

PRO bye_events, event
print, 'goodbye'
Widget_Control, event.top, /Destroy
END

PRO MS_reset_events, event
Widget_Control, event.top, Get_UValue=info
wset, info.wid
erase
*info.ms_index_list=-1
Widget_Control, info.MSID, Set_List_Select=-1
Widget_Control, event.top, Set_UValue=info
END

PRO reset_events, event
Widget_Control, event.top, Get_UValue=info
Widget_Control, info.xminID, Set_Value=-3
Widget_Control, info.xmaxID, Set_Value=3
Widget_Control, info.yminID, Set_Value=-3
Widget_Control, info.ymaxID, Set_Value=3
info.lim_change = 1
;
; Send the MS_events an event to make the new plot
ms_index_list = *info.ms_index_list
MSID =info.MSID
; The tlb needs its user value restored
Widget_Control, event.top, Set_UValue=info
; Construct the pseudo event structure for MS 
pseudo_event = {ID:MSID, Top:event.top, Handler:0L, Index:ms_index_list, Clicks:1L}
; Call MS_events with the pseudo event
MS_events, pseudo_event
;
END

PRO zoomout_events, event
Widget_Control, event.top, Get_UValue=info
;
expand_factor = 2.0
Widget_Control, info.xminID, Get_Value=xmin
Widget_Control, info.xmaxID, Get_Value=xmax
Widget_Control, info.yminID, Get_Value=ymin
Widget_Control, info.ymaxID, Get_Value=ymax
x_ran =  (xmax - xmin)
y_ran =  (ymax - ymin)
x_mid =  (xmax + xmin)*0.5
y_mid =  (ymax + ymin)*0.5
xmin_new = x_mid - (x_ran * expand_factor)*0.5
xmax_new = x_mid + (x_ran * expand_factor)*0.5
ymin_new = y_mid - (y_ran * expand_factor)*0.5
ymax_new = y_mid + (y_ran * expand_factor)*0.5
;
Widget_Control, info.xminID, Set_Value=xmin_new
Widget_Control, info.xmaxID, Set_Value=xmax_new
Widget_Control, info.yminID, Set_Value=ymin_new
Widget_Control, info.ymaxID, Set_Value=ymax_new
info.lim_change = 1

; Send the MS_events an event to make the new plot
ms_index_list = *info.ms_index_list
MSID =info.MSID
; The tlb needs its user value restored
Widget_Control, event.top, Set_UValue=info
; Construct the pseudo event structure for MS 
pseudo_event = {ID:MSID, Top:event.top, Handler:0L, Index:ms_index_list, Clicks:1L}
; Call MS_events with the pseudo event
MS_events, pseudo_event
;
END


PRO erase_events, event
Widget_Control, event.top, Get_UValue=info
Wset, info.wid
erase
Widget_Control, event.top, Set_UValue=info
END


PRO MS_events, event
Widget_Control, event.top, Get_UValue=info
;
; Check that there are valid line selections for X1,X2,Y1,Y2
x1 = *info.x1
x2 = *info.x2
y1 = *info.y1
y2 = *info.y2

; Send a warning message to the screen and return if any of the
; line selections widgets have not been set
IF ((x1[0] EQ -1) OR (x2[0] EQ -1) OR (y1[0] EQ -1) OR (y2[0] EQ -1))  THEN BEGIN
warning = dialog_message(['Emission lines must be selected before plotting a model grid.',  $
                         'Choose emission lines in each of the Y1, Y2, X1, X2 lists, or ',  $
                         'select pre-defined line ratios in the Diagrams munu list'],      $
                         dialog_parent=info.MSID)
MS_reset_events, event
RETURN
ENDIF
;
;
Widget_Control, event.id, Get_UValue=ms_files
Widget_Control, info.xminID, Get_Value=xmin
Widget_Control, info.xmaxID, Get_Value=xmax
Widget_Control, info.yminID, Get_Value=ymin
Widget_Control, info.ymaxID, Get_Value=ymax
xra_vec = [xmin,xmax]
yra_vec = [ymin,ymax]
;
; Update ms_index_list with the newly selected model set index 
ms_index_list = [*info.ms_index_list,event.index] 
;
; Make sure that ms_index_list has only unique values. This
; is importnant in the case where a model set is selected multiple times
ms_index_list=ms_index_list[uniq(ms_index_list,sort(ms_index_list))]
;
; The initial value of ms_index_list is -1, and the first selections
; will still have this value as the first element of ms_index_list
; array. If this is the case, we get rid of this value.
IF ms_index_list[0] EQ -1 then ms_index_list = ms_index_list[1:*]

; If there is more than one model set selected, make sure the 
; MS list widget correctly shows this.
;
; Note that the /multiple keyword of the list widget is set, so
; that the list can show multiple selected items. Selecting more
; than one item with one click does not appear to work properly,
; so we we require that multiple items be selected with individual
; clicks that generate an event each time, and a ms_index_list
; is a cumulative list of those items. The list can only be
; reliably reset with the MS_reset widget.
;   
IF n_elements(ms_index_list) GT 1 then Widget_Control, event.id, Set_List_Select=ms_index_list

; Here are some plotting parameters needed before the plot
; command is issued in the main loop below. These plotting
; parameters are better set here, out of the loop, because 
; they only need to be set once. This is particularly true
; for the postscript file settings, because we (ususally)
; want only a single postscript file to be generated at a
; time.
; Note that the postscipt file name is hardcoded as tmp.ps
; (In future a file-chooser widget might be better) 

; Erase the window if the limits have been changed
IF info.lim_change EQ 1  THEN erase

lab_col = ['0000FF'xL,'FFFF00'xL]
lab_col_ps = [0,0]
 
xthick=1
ythick=1
IF info.ps THEN BEGIN
ps_start,'tmp.ps',24,20,/col
xthick = 3
ythick = 3
lab_col = lab_col_ps
ENDIF ELSE Wset, info.wid

;-------------------------------------------------------
; Here is the main loop running over all the elements of
; ms_index_list, that is all of the selected model sets
;
FOR msid = 0,n_elements(ms_index_list)-1 DO BEGIN
ms_index = ms_index_list[msid]

;print, 'ms_index', ms_index
; Get the names of the relevant files from the ms_files array
model_set_save_files = ms_files[ms_index,*]
; Get rid of the empty elements 
model_set_save_files = model_set_save_files[where(strlen(model_set_save_files) GT 0)]

; Define a string variable that will contain a single element being 
; the list of structure names. This will be used to create a structure
; using the IDL execute function. 
structures_string = ''
;
; Define a string array that will hold a string of the IDL
; command that will reproduce the plot.
restore_command_strings=STRARR(n_elements(model_set_save_files))

; loop over all the selected save files
;
FOR i=0,n_elements(model_set_save_files)-1 DO BEGIN
restore, filename=info.save_dir+model_set_save_files[i]
restore_command_strings[i] = "restore, filename='" + info.save_dir+model_set_save_files[i] + "'"
struct_name=strmid(model_set_save_files[i],0,strlen(model_set_save_files[i])-5)
structures_string = structures_string + ', m' + strcompress(string(i),/remove_all) + ':' +struct_name
ENDFOR
tmp = execute('m_sets_struct = {' + strmid(structures_string,1) + '}')

; The velocity range to be plotted is set here
vra=[200,5000]
vlabs=*info.vlabs
blabs=*info.blabs
p_grid =*info.p_grid
p_command = *info.p_command
plot_ct = *info.plot_ct

; ************************************************************
; This is the most important line in the whole program,
; it draws the plot.
plot_grid, LINE1=*info.x1, LINE2=*info.x2, LINE3=*info.y1 $
           ,LINE4=*info.y2, MODEL_SET_STRUCT=m_sets_struct $
           ,vra=vra, xra=xra_vec, yra=yra_vec $
           ,pos=[0.1,0.1,0.75,0.9] $
           , XTHICK=xthick, YTHICK=ythick $
           , p_grid=p_grid $
           , plot_ct=plot_ct $
           , bctab = 1 $
           , vctab = 3 $
           ,/auto_title, vlabs=vlabs, blabs=blabs, lab_col=lab_col,/vel_lim
; ************************************************************


; If the p_command flag is set, then print out an IDL command that will
; reproduce the current plot
IF p_command THEN BEGIN

; Here we construct the strings necessary for making a IDL command
; which will produce the same result as drawn above, so that
; the plot can be reproduced (and tweaked or included in scripts etc.)
; without this graphical widget interface.
;
ax1=strcompress(strjoin(string(*info.x1),','),/remove_all)
ax2=strcompress(strjoin(string(*info.x2),','),/remove_all)
ay1=strcompress(strjoin(string(*info.y1),','),/remove_all)
ay2=strcompress(strjoin(string(*info.y2),','),/remove_all)
if n_elements(*info.x1) GT 1 then ax1 = '[' + ax1 + ']'
if n_elements(*info.x2) GT 1 then ax2 = '[' + ax2 + ']'
if n_elements(*info.y1) GT 1 then ay1 = '[' + ay1 + ']'
if n_elements(*info.y2) GT 1 then ay2 = '[' + ay2 + ']'

; Construct the axis titles.
; This block of code is almost identical to a similar block in
; plot_grid.pro . It is reproduced here, because it doesn't seem
; sensible to output it from plotgrid.pro.  
;
model = m_sets_struct.(0)
atom = model.L.A
ion_state= model.L.I
w = model.L.W
;
a=*info.x1 & b=*info.x2 & d=*info.y1 & e=*info.y2
a_wave ='' 
FOR i=0,N_ELEMENTS(a)-1 DO BEGIN
a_wave = a_wave + ' ' +STRCOMPRESS(STRING(w[a[i]]), /REMOVE_ALL)
ENDFOR
b_wave ='' 
FOR i=0,N_ELEMENTS(b)-1 DO BEGIN
b_wave = b_wave + ' ' +STRCOMPRESS(STRING(w[b[i]]), /REMOVE_ALL)
ENDFOR
d_wave ='' 
FOR i=0,N_ELEMENTS(d)-1 DO BEGIN
d_wave = d_wave + ' ' +STRCOMPRESS(STRING(w[d[i]]), /REMOVE_ALL)
ENDFOR
e_wave ='' 
FOR i=0,N_ELEMENTS(e)-1 DO BEGIN
e_wave = e_wave + ' ' +STRCOMPRESS(STRING(w[e[i]]), /REMOVE_ALL)
ENDFOR
x_title = atom[a[0]]+ion_state[a[0]]+a_wave+' / '+atom[b[0]]+ion_state[b[0]]+b_wave
y_title = atom[d[0]]+ion_state[d[0]]+d_wave+' / '+atom[e[0]]+ion_state[e[0]]+e_wave

; Print out command line strings to produce the same plot
print, '; ==========================================================================='
print, ';'
print, ';  The following commands may be entered at the IDL command' 
print, ';  line to reproduce this plot'
print, ';'
print, ';  Y: ' + y_title
print, ';  X: ' + x_title
print, ';'
print, '; Restore models from IDL save files'
print, '; (each save file contains a structure of the same name)'
FOR u=0,n_elements(restore_command_strings)-1 DO print, restore_command_strings[u]
print, ';'
print, '; Construct a single structure of the restored model structures'
print, 'm_sets_struct = {' + strmid(structures_string,1) + '}'
print, ';'
print,'; Call the plot_grid procedure to make the plot' 
print,  'plot_grid,LINE1=' + ax1  + $
                  ',LINE2=' + ax2  + $
                  ',LINE3=' + ay1  + $
                  ',LINE4=' + ay2  + $
                  ',MODEL_SET_STRUCT=m_sets_struct' + $
                  ',vra=[' + strcompress(string(vra[0]),/remove_all) +','+ strcompress(string(vra[1]),/remove_all) +']'+ $
                  ',xra=['+ strcompress(string(xmin),/remove_all) +','+ strcompress(string(xmax),/remove_all) + $
                  '],yra=['+ strcompress(string(ymin),/remove_all) +','+ strcompress(string(ymax),/remove_all) +']' +$
                  ',pos=[0.1,0.1,0.75,0.9]' +$
                  ',/auto_title,/vel_lim' + $
                  ',bctab=1,vctab=3,plot_ct=1' + $
                  ',vlabs=[' + strjoin(strcompress(string(vlabs),/remove_all),',') +']' + $
                  ',blabs=['+ strcompress(string(blabs),/remove_all) + ']' + $
                  ',lab_col=[' + strcompress(string(lab_col[0]),/remove_all) +','+ $
                   strcompress(string(lab_col[1]),/remove_all) +']'
print, ';'
print, '; ==========================================================================='
ENDIF


; This is the end of the main loop over all the elements of ms_index_list.
ENDFOR ;------------------------------------------------------------------

IF info.ps THEN BEGIN
ps_end
info.ps=0
ENDIF

IF NOT info.ps THEN BEGIN
; Now that the plot is drawn, we keep a copy of the graphics window, 
; which is needed for the rubber-band-box selection function for
; zooming in, and redrawing the plot with different x and y limits
; (as is done in draw_events)
Window, 1, XSize=600, YSize=500, /Pixmap
info.pixIndex = 1
Device, Copy = [0, 0, 600, 500, 0, 0, info.wid]
;
; reset the graphics window to the main widget graphics window
wset, info.wid
ENDIF

info.lim_change=0
*info.ms_index_list = ms_index_list 
Widget_Control, event.top, Set_UValue=info, /No_Copy

END ; of MS_events ===========================================================



PRO draw_events, event
; This event handler continuously draws and erases the zoom box until it
; receives an UP event from the draw widget. Then it turns draw widget
; motion events OFF.

; Get the info structure out of the top-level base.
Widget_Control, event.top, Get_UValue=info, /No_Copy

; What type of an event is this?
possibleEventTypes = [ 'DOWN', 'UP', 'MOTION', 'SCROLL' ]
thisEvent = possibleEventTypes(event.type)

CASE thisEvent OF

   'DOWN': BEGIN
    ; Set the static corners of the box to current
    ; cursor location.
    info.xs = event.x
    info.ys = event.y
    ; Turn draw MOTION events ON.
    Widget_Control, event.id, Draw_Motion_Events=1
    ENDCASE ; of DOWN

   'UP': BEGIN
    WSet, info.wid
    ; Turn motion events off.
    Widget_Control, event.id, Draw_Motion_Events=0
    ;
    ; Get the limits for the new plot 
    event.x = 0 > event.x < (599)
    event.y = 0 > event.y < (499)
    x = [info.xs, event.x]
    y = [info.ys, event.y]
    top_left_dev = [min(x),max(y)]
    bot_right_dev = [max(x),min(y)]
    top_left = convert_coord(top_left_dev,/device,/to_data)
    bot_right = convert_coord(bot_right_dev,/device,/to_data)
    ;
    ; Make sure the user didn't just click in the window.
    IF info.xs EQ event.x OR info.ys EQ event.y THEN BEGIN
       Widget_Control, event.top, Set_UValue=info, /No_Copy
       RETURN
    ENDIF
    ; Set the new limits
     Widget_Control, info.xminID, Set_Value=top_left[0]
     Widget_Control, info.xmaxID, Set_Value=bot_right[0]
     Widget_Control, info.yminID, Set_Value=bot_right[1]
     Widget_Control, info.ymaxID, Set_Value=top_left[1]
     ; Set the flag to say that the limits have changed
     info.lim_change=1
     ;
     ; Send the MS_events an event to make the new plot
     ms_index_list = *info.ms_index_list
     MSID =info.MSID
     ;ms_pi_index = *info.ms_pi_index
     ;MS_piID =info.MS_piID
     ; The tlb needs its user value restored
     Widget_Control, event.top, Set_UValue=info, /No_Copy
     ; Construct the pseudo event structure for MS 
     pseudo_event = {ID:MSID, Top:event.top, Handler:0L, Index:ms_index_list, Clicks:1L}
     ; Call MS_events with the pseudo event
     MS_events, pseudo_event
     ;
     RETURN
     ;
     ENDCASE ; of UP
   
    'MOTION': BEGIN
    ; Most of the action in this event handler occurs here while we are waiting
    ; for an UP event to occur. As long as we don't get it, keep erasing the
    ; old zoom box and drawing a new one.
    ;
    ; Erase the old zoom box.
    WSet, info.wid
    Device, Copy = [0, 0, 600, 500, 0, 0, info.pixIndex]
    ;
    ; Update the dynamic corner of the zoom box to the current cursor location.
    info.xd = event.x
    info.yd = event.y
    PlotS, [info.xs, info.xs, info.xd, info.xd, info.xs], $
       [info.ys, info.yd, info.yd, info.ys, info.ys], /Device
    ;
    ENDCASE ; of MOTION

; This the end of the case statements
ENDCASE 
;
Widget_Control, event.top, Set_UValue=info, /No_Copy
END ; of draw_events =========================================================


PRO  ps_events, event
Widget_Control, event.top, Get_UValue=info
info.ps=1
; set system graphics variables for plot
orig_charthick = !P.CHARTHICK
orig_axisthick = !P.THICK
!P.CHARTHICK=3
!P.THICK = 1

; Send the MS_events an event
ms_index_list = *info.ms_index_list
MSID =info.MSID

;The tlb needs its user value restored
Widget_Control, event.top, Set_UValue=info, /No_Copy
; Construct the pseudo event structure
pseudo_event = {ID:MSID, Top:event.top, Handler:0L, Index:ms_index_list, Clicks:1L}
; Call MS_events with the pseudo event
MS_events, pseudo_event
; 
!P.CHARTHICK=orig_charthick
!P.THICK = orig_axisthick
;
END ; of ps_events ===========================================================


PRO Shockplot_Cleanup,top
; The purpose of this procedure is to clean up all things that use
; memory upon the closure of the widget.
; This procedure is called when the top-level
; base widget is destroyed.
 On_Error,2
 Widget_Control, top, Get_UValue=info, /No_Copy
 IF N_Elements(info) EQ 0 THEN RETURN
  ptr_free,info.x1,info.x2,info.y1,info.y2
  ptr_free,info.p_grid,info.vlabs,info.blabs,info.p_command,info.plot_ct
  ptr_free,info.ms_index_list, info.full_line_list
 RETURN
END ; of Shockplot_Cleanup ===================================================


PRO shockplot_event, event
Widget_Control, event.top, Get_UValue=info

; listen to compound widgets
IF ((event.id EQ info.xminID) OR (event.id EQ info.xmaxID) OR $
    (event.id EQ info.yminID) OR (event.id EQ info.ymaxID))  THEN BEGIN
        ;
        ; print, 'Someone pressed the slider'
        info.lim_change=1
        ; get widget IDs and relevant selection indexes
        ms_index_list = *info.ms_index_list
        MSID =info.MSID
        ;The tlb needs its user value restored
        Widget_Control, event.top, Set_UValue=info, /No_Copy
        ;
        ; Send the MS_events an event
        ; Construct the pseudo event structure
        pseudo_event = {ID:MSID, Top:event.top, Handler:0L, Index:ms_index_list, Clicks:1L}
        ; Call MS_events with the pseudo event
        MS_events, pseudo_event
        ;
        RETURN
ENDIF

Widget_Control, event.top, Set_UValue=info
END ; of shockplot_events ====================================================


PRO shockplot
; This is the widget definition module, it sets up the layout of the
; tool, and also does all the preliminary set up
 
save_dir = 'save_files/'
save_files = findfile(save_dir)

; Define some null string arrays which will later contain the 
; names of model structures
struct_names  =strarr(n_elements(save_files))
struct_names_2=strarr(n_elements(save_files))

; Each .save file contains a single structure whose name
; is the same as the root of the .save file name. That is,
; the file M_n1_b1_sp.save contains the structure m_n1_b1_sp

; The following manipulations of the file names get a bit 
; involved, and there's probably a simpler way to do it, but
; essentially it is making a set of string arrays containing 
; various groupings of the original filenames and structures
; they contain.

; Loop over each filename, and put structure name into the
; struct_name and struct_names_2 arrays. 
FOR i=0,n_elements(save_files)-1 DO BEGIN
    ; strip off the final 5 characters (.save) to get names like M_n1_b1_sp
    struct_names[i] = strmid(save_files[i],0,strlen(save_files[i])-5)
    ; get the characters after the last _ in struct_names, i.e. _sp
    struct_names_2[i] = strmid(struct_names[i],strpos(struct_names[i],'_',/reverse_search))
ENDFOR

; define struct_names_1 as the first character of struct_names and
; the final chars. I.e. M_sp . This construction will provide a convenient
; grouping of the _s and _sp subsets
struct_names_1 = strmid(struct_names,0,1) + struct_names_2

; define ms_names as the sorted list of unique struct_names_1
; i.e. ['M_p', 'M_s', 'M_sp', ... ]
; This is useful for specifying all the model sets in the MS widget 
ms_names = struct_names_1[uniq(struct_names_1, sort(struct_names_1))]


; Define an array with as many rows as there are struct_names_1
; values, i.e. all the model sets [M_s, M_sp,....U_s,...]
; Then fill each row with the names of the save files in that group.
; The array allows for 50 filenames in a model set, but typically
; there are less than 10.
;
ms_files = strarr(n_elements(struct_names_1),50)
FOR i=0,n_elements(ms_names)-1 DO BEGIN
ms_files_vec = save_files[where( (strmid(save_files,0,1) EQ strmid(ms_names[i],0,1)) $
                          AND (struct_names_2 EQ strmid(ms_names[i],1)))]
;print, ms_names[i]
;print, ms_files_vec
ms_files[i,0:n_elements(ms_files_vec)-1] = ms_files_vec
ENDFOR

; Summarising all these file name and structure name arrays
; struct_names   : 1d array of structure names
; struct_names_2 : 1d array of '_s','_sp', '_p' strings
; struct_names_1 : 1d array of 'M_s', strings
; 
; ms_names       : 1d array of uniq model set names 'M_s', 'N_s'
; ms_files       : 2d array, with as many rows as elements in ms_names
;                  containing all the filenames of a given model set
;                  Note that this is set as the user value of the MS widget
; i.e. ms_files[0,*] =
;        A_n1_b05_p.save
;        A_n1_b10_p.save
;        A_n1_b1_p.save
;        A_n1_b2_p.save
;        A_n1_b4_p.save
;        A_n1_b5_p.save
;        A_n1_be_p.save


; Restore the first file, and rename it into a structure called model
; which will be used for setting various initial values 
restore, filename=save_dir+save_files[0]
; Here we use the execute command because it is difficult to get the 
; the structre name to be interpretted as the actual structure, not
; just a string
tmp = execute('model='+struct_names[0])


; Set up the layout of the widget
tlb  =Widget_Base(Row=1, mbar=menu, $
      title='SHOCKPLOT')
c1b  =Widget_Base(tlb,Row=4)
c2b  =Widget_Base(tlb,Row=4,/grid_layout)
c2r1b =Widget_Base(c2b,Row=2)
c2r2b =Widget_Base(c2b,Row=2)
c2r3b =Widget_Base(c2b,Row=2)
c2r4b =Widget_Base(c2b,Row=2)
c3b  = Widget_Base(tlb,Row=20)
c4b  = Widget_Base(tlb,Col=1)
draw = Widget_Draw(c4b, Xsize=600, Ysize=500, Button_Events=1, Event_Pro='draw_events')
c4br= Widget_Base(c4b,Col=2)
c4br2 = Widget_Base(c4br,col=2)
c4br3 = Widget_Base(c4br,col=3)


; Define the items in the menu bar at the top of the window

; The File menu button
file = widget_button(menu, value='File', /menu, event_pro='file_menu_handler')
psID  = Widget_Button(file,   Value='PRINT (to file tmp.ps)      ', UValue='print')
byeButton = Widget_Button(file, Value='EXIT', UValue='exit' )

; The Options menu button
options = widget_button(menu, value='Options', /menu, event_pro='options_menu_handler')
velLabels = Widget_Button(options, Value='Velocity Labels', $
                          UValue=0,/checked_menu)
magLabels = Widget_Button(options, Value='Magnetic Field Labels', $
                          UValue=0,/checked_menu) 
p_gridButton = Widget_Button(options, Value='Print Grid Values', $
                          UValue=0,/checked_menu) 

p_commandButton = Widget_Button(options, Value='Print IDL Command', $
                          UValue=0,/checked_menu) 

plot_ctButton = Widget_Button(options, Value='Colour Key', $
                          UValue=1,/checked_menu) 
; p_grid
; color tables bctab, clip_bct, vctab, clip_vct
; print command
; vlim 
; vra ?

; The Line list menu button
linelist_button =  widget_button(menu, value='LineList', /menu, event_pro='linelist_menu_handler')
full = Widget_Button(linelist_button, Value='Full ', Uvalue='full')
UV_Opt_IR = Widget_Button(linelist_button, Value='Selected  UV, Optical and IR lines', Uvalue='s1')
wave_range1 = Widget_Button(linelist_button, Value='1000-2000 Angstrom', Uvalue='wr1')
wave_range2 = Widget_Button(linelist_button, Value='2000-3000 Angstrom', Uvalue='wr2')
wave_range3 = Widget_Button(linelist_button, Value='3000-4000 Angstrom', Uvalue='wr3')
wave_range4 = Widget_Button(linelist_button, Value='4000-5000 Angstrom', Uvalue='wr4')
wave_range5 = Widget_Button(linelist_button, Value='5000-6000 Angstrom', Uvalue='wr5')
wave_range6 = Widget_Button(linelist_button, Value='6000-7000 Angstrom', Uvalue='wr6')
wave_range7 = Widget_Button(linelist_button, Value='7000-8000 Angstrom', Uvalue='wr7')
wave_range8 = Widget_Button(linelist_button, Value='8000-9000 Angstrom', Uvalue='wr8')
wave_range9 = Widget_Button(linelist_button, Value='9000-10000 Angstrom', Uvalue='wr9')
wave_range10 = Widget_Button(linelist_button, Value='1-2 micron', Uvalue='wr10')
wave_range11 = Widget_Button(linelist_button, Value='2-5 micron', Uvalue='wr11')
wave_range12 = Widget_Button(linelist_button, Value='5-10 micron', Uvalue='wr12')
wave_range13 = Widget_Button(linelist_button, Value='10-25 micron', Uvalue='wr13')
wave_range14 = Widget_Button(linelist_button, Value='25-100 micron', Uvalue='wr14')
wave_range15 = Widget_Button(linelist_button, Value='100-200 micron', Uvalue='wr15')
wave_range16 = Widget_Button(linelist_button, Value='200-300 micron', Uvalue='wr16')

; The Diagrams menu button
std_diags_menu  = widget_button(menu, value='Diagrams', /menu,  event_pro='std_diags_menu_handler')
; see the definition of all the standard diagrams below where each is defined
; as a button widget in this menu

; The Help menu button
help_menu = widget_button(menu, value='Help', /menu, event_pro='help_menu_handler')
help_about =  widget_button(help_menu, value='About', uvalue='about' )
help_instr =  widget_button(help_menu, value='Instructions', uvalue='instructions' )
help_readme =  widget_button(help_menu, value='Readme file', uvalue='readme' )

; Definition of all the standard diagrams. (These can be re-ordered)
;
std_diag1 =  Widget_Button(std_diags_menu, Value='[OIII]/H-beta vs. [NII]/H-alpha', $
                           UValue={x1:[1432,1435], x2:1434, y1:1400, y2:1393 } )
;......................................................................................
std_diag2 =  Widget_Button(std_diags_menu, Value='[OIII]/H-beta vs. [SII]/H-alpha', $
                           UValue={x1:[1439,1440], x2:1434, y1:1400, y2:1393 } )
;......................................................................................
std_diag3 =  Widget_Button(std_diags_menu, Value='[OIII]/H-beta vs. [OII]3727/[OIII]5007', $
                           UValue={x1:[1345,1346], x2:1400, y1:1400, y2:1393 })
;......................................................................................
std_diag4 =  Widget_Button(std_diags_menu, Value='[OIII]/H-beta vs. [OI]/H-alpha', $
                           UValue={x1:1422, x2:1434, y1:1400, y2:1393 })
;......................................................................................
std_diag5 =  Widget_Button(std_diags_menu, Value= 'HeII/H-beta vs. [OIII]/H-beta'  , $
                           UValue= {x1:1400, x2:1393, y1:1384, y2:1393 } )
;......................................................................................
std_diag6 =  Widget_Button(std_diags_menu, Value= '[OIII]/H-beta vs. CIV/HeII' , $
                           UValue= {x1:[1209,1210], x2:1218, y1:1400, y2:1393})
;......................................................................................
std_diag7 =  Widget_Button(std_diags_menu, Value='[OIII]/H-beta vs. CIV/CIII]'  , $
                           UValue=  {x1:[1209,1210],x2:[1247,1248],y1:1400,y2:1393} )
;......................................................................................
std_diag8 =  Widget_Button(std_diags_menu, Value= 'CII]/CIII] vs. CIV/CIII]' , $
                           UValue= {x1:[1209,1210],x2:[1247,1248],y1:1277,y2:[1247,1248]})
;......................................................................................
std_diag9 =  Widget_Button(std_diags_menu, Value= '[OIII]/H-beta vs. [NeV]/[NeIII]' , $
                           UValue=     {x1:1331, x2:1354, y1:1400, y2:1393}          )
;......................................................................................
std_diag10 =  Widget_Button(std_diags_menu, Value=  '[OIII]/H-beta vs. NIII1750/NIII]991' , $
                           UValue=  {x1:[1226,1227,1228,1230,1231],x2:1112,y1:1400,y2:1393})
;......................................................................................
std_diag11 =  Widget_Button(std_diags_menu, Value= '[OIII]/H-beta vs. CIII]1909/CIII]997' , $
                           UValue=      {x1:[1247,1248],x2:1109,y1:1400,y2:1393}         )
;......................................................................................
std_diag12 =  Widget_Button(std_diags_menu, Value=  '[OIII]4363/[OIII]5007 vs.[OIII5007]/H-beta' , $
                           UValue=  {x1:1400,x2:1393,y1:1377,y2:1400}            )
;......................................................................................
std_diag13 =  Widget_Button(std_diags_menu, Value=  '[NeV]3426/[NeIII3868] vs. [NeIII]3868/[OII]3727' , $
                           UValue=  {x1:1354,x2:[1345,1346],y1:1331,y2:1354}             )
;......................................................................................
std_diag14 =  Widget_Button(std_diags_menu, Value= 'OIII5007/Hb vs. OIII4363/OIII5007' , $
                           UValue=   {x1:1377,x2:1400,y1:1400,y2:1393}            )
;......................................................................................
std_diag15 =  Widget_Button(std_diags_menu, Value='[OIII]/H-beta vs. HeII/H-beta'  , $
                           UValue=  {x1:1384, x2:1393, y1:1400, y2:1393 }            )
;......................................................................................
std_diag16 =  Widget_Button(std_diags_menu, Value=  '[OIII]/H-beta vs. [NeV]3426/H-beta' , $
                           UValue=   {x1:1384,x2:1393,y1:1400,y2:1393}            )
;......................................................................................
std_diag17 =  Widget_Button(std_diags_menu, $
              Value='HeII/Lya vs. Lya/CIV  (Whittle ApJ 2005, 129, 104, fig 6a)'  , $
                           UValue=  {x1:1157,x2:[1209,1210],y1:1218,y2:1156 }             )
;......................................................................................
std_diag18 =  Widget_Button(std_diags_menu, $
              Value= 'Hb/Lya vs. [OIII]/[OII]  ( Whittle ApJ 2005, 129, 104, fig 6b)'  , $
                           UValue=  {x1:1400,x2:[1345,1346],y1:1393,y2:1156 }             )
;......................................................................................
std_diag19 =  Widget_Button(std_diags_menu, Value= '[OIII]/H-beta vs. NV1240/CIV'  , $
                           UValue=  {x1:[1160,1163],x2:[1210,1209],y1:1400,y2:1393}            )
;......................................................................................
std_diag20 =  Widget_Button(std_diags_menu, Value='NeV 14.3um/NeII 12.8um vs. NeIII 15.5um/NeII 12.8um'  , $
                           UValue=  {x1:1694,x2:1686,y1:1690,y2:1686}              )
;......................................................................................
std_diag21 =  Widget_Button(std_diags_menu, Value= 'NeV 14.3um/NeII 12.8um vs SIV 10.5um / SIII 33.6um' , $
                           UValue=     {x1:1675,x2:1715,y1:1690,y2:1686}           )
;......................................................................................
std_diag22 =  Widget_Button(std_diags_menu, Value=   'CIII]/CII] vs [NeIII]/[NeV]' , $
                           UValue=    {x1:1354,x2:1331,y1:[1247,1248],y2:1277}           )
;......................................................................................

; extract the line list from the model structure of the initial model
; (this really should be done by reading in from a dedicated file... ...anyhoo...)
full_line_list = model.l
; construct a string array of the array names for display in the line
; selection widgets
lines = STRCOMPRESS(model.l.a +' '+ model.l.i + string(model.l.w))
; construct an integer array of the indexes of the lines (which is the way the
; lines are identified explicitly in the plotting routines (plot_grid)
nlines = indgen(N_ELEMENTS(lines))

; Specify the initial set of lines to be listed 
;
line_selection =WHERE(model.l.w GT 500) 

options = lines[line_selection]
option_vals = nlines[line_selection]
;
Y1_t = Widget_Label(c1b, Value='Y1')
Y1 = Widget_List(c1b, Value=options, Ysize=10, xsize=20, $
      Frame=1, UValue=option_vals, Event_Pro='Y1_events', /multiple)
Y2_t = Widget_Label(c1b, Value='Y2')
Y2 = Widget_List(c1b, Value=options, Ysize=10, xsize=20, $
      Frame=1, UValue=option_vals, Event_Pro='Y2_events', /multiple)
X1_t = Widget_Label(c1b, Value='X1')
X1 = Widget_List(c1b, Value=options, Ysize=10, xsize=20, $
      Frame=1, UValue=option_vals, Event_Pro='X1_events', /multiple)
X2_t = Widget_Label(c1b, Value='X2')
X2 = Widget_List(c1b, Value=options, Ysize=10, xsize=20,$
      Frame=1, UValue=option_vals, Event_Pro='X2_events', /multiple)


Y1_selected = Widget_List(c2r1b, Value=[' ',' ',' '],UValue={l_list_index:[-1], $
                          option_vals_index:[-1]},Ysize=8,xsize=20)

Y1_clear = Widget_Button(c2r1b, Value='Clear Y1', Event_Pro='Clear_Y1_events',ysize=25)

Y2_selected = Widget_List(c2r2b, Value=[' ',' ',' '],UValue={l_list_index:[-1], $
                          option_vals_index:[-1]},Ysize=8,xsize=20)

Y2_clear = Widget_Button(c2r2b, Value='Clear Y2', Event_Pro='Clear_Y2_events',ysize=26)

X1_selected = Widget_List(c2r3b, Value=[' ',' ',' '],UValue={l_list_index:[-1], $
                          option_vals_index:[-1]},Ysize=8,xsize=20)

X1_clear = Widget_Button(c2r3b, Value='Clear X1', Event_Pro='Clear_X1_events',ysize=26)

X2_selected = Widget_List(c2r4b, Value=[' ',' ',' '],UValue={l_list_index:[-1], $
                          option_vals_index:[-1]},Ysize=8,xsize=20)

X2_clear = Widget_Button(c2r4b, Value='Clear X2', Event_Pro='Clear_X2_events',ysize=26)

;
; Human readable names for the shock model sets
ms_names_text = ms_names
;
ms_names_text[where(ms_names_text EQ 'M_p')]  = 'n=1    abn=Sol Precursor'
ms_names_text[where(ms_names_text EQ 'M_s')]  = 'n=1    abn=Sol Shock'
ms_names_text[where(ms_names_text EQ 'M_sp')] = 'n=1    abn=Sol Shock+Precursor'

model_names=struct_names

; The model set selection widget
MS  = Widget_List(c3b, Value=ms_names_text, Ysize=39,xsize=35, $
      Frame=2, UValue=ms_files, Event_Pro='MS_events', /multiple)

; A button to reset the model set selection to none
MS_reset = Widget_Button(c3b, Value='RESET SHOCK MODEL', Event_Pro='MS_reset_events',ysize=26)

; Define the slider widgets for changing the plot range, and which 
; are also called by the stretch box procedures 
xra_default = [-3,3]
yra_default = [-3,3]
slider_min=-20
slider_max=20
min=-3
max=3
;
xminID = CW_FSLIDER(c4br2, minimum=slider_min, maximum=slider_max, drag=0, $
                        Value=min, UValue='min', Title='X MIN', Edit=1)

yminID = CW_FSLIDER(c4br2, minimum=slider_min, maximum=slider_max, drag=0, $
                        Value=min, UValue='min', Title='Y MIN', Edit=1)


xmaxID = CW_FSLIDER(c4br2, minimum=slider_min, maximum=slider_max, drag=0, $
                        Value=max, UValue='max', Title='X MAX', Edit=1)

ymaxID = CW_FSLIDER(c4br2, minimum=slider_min, maximum=slider_max, drag=0, $
                        Value=max, UValue='max', Title='Y MAX', Edit=1)

; The zoom out button
ZoomOutID = Widget_Button(c4br3, Value='ZOOM OUT', Event_Pro='zoomout_events')

; Button to reset the plot range to default values
resetID = Widget_Button(c4br3, Value='Default Limits', Event_Pro='reset_events')

; An example button that can be defined by the use
userdef1ID = Widget_Button(c4br3, Value='User Defined',sensitive=0)

; The Exit button
byeButton = Widget_Button(c4br3, Value='EXIT', Event_Pro='bye_events')

; Realize the top level base
Widget_Control, tlb, /Realize
;
; Get the window index of the plotting window
Widget_Control, draw, Get_Value=windowIndex
Wset, windowIndex
;

; define the info structure
info = { x1:Ptr_new(-1), x2:Ptr_new(-1), y1:Ptr_new(-1), y2:Ptr_new(-1), wid:windowIndex, $
         x1_id:X1,  x1_selected:X1_selected, $
         x2_id:X2,  x2_selected:X2_selected, $
         y1_id:Y1,  y1_selected:Y1_selected, $
         y2_id:Y2,  y2_selected:Y2_selected, $
         x1_clear:X1_clear, x2_clear:X2_clear, y1_clear:Y1_clear, y2_clear:Y2_clear, $
         xminID:xminID, xmaxID:xmaxID, yminID:yminID, ymaxID:ymaxID, $
         MSID:MS,           $
         vlabs:Ptr_new(0),  $
         blabs:Ptr_new(0),  $
         p_grid:Ptr_new(0),          $
         p_command:Ptr_new(0),          $
         plot_ct:Ptr_new(1),          $
         lim_change:0,      $  ; flag which is set when plot limits change
         l_list:lines,      $
         save_files:save_files,      $
         save_dir:save_dir,          $
         ms_index_list:Ptr_new(-1),  $
         full_line_list: Ptr_new(full_line_list), $
         fullID:full, $
         xs:0, $                ; X static corner of the zoom box.
         ys:0, $                ; Y static corner of the zoom box.
         xd:0, $                ; X dynamic corner of the zoom box.
         yd:0, $                ; Y dynamic corner of the zoom box.
         ps:0, $                ; flag for generating postscript files
         pixIndex:0 }            ; The pixmap window index number.

;
Widget_control, tlb, Set_UValue=info
Widget_control, plot_ctButton,  Set_Button=1
XManager, 'shockplot', tlb,cleanup='shockplot_cleanup'
;
END ; of shockplot ===========================================================

